using System;
using System.Collections.Generic;
using com.agilefaqs.rft.domain;
using com.agilefaqs.rft.suggestions.strategies;
using NUnit.Framework;
using Rhino.Mocks;

namespace com.agilefaqs.rft.suggestions
{
    [TestFixture]
    public class PartialAcceptanceTest
    {
        internal class AnonymousClassRandomNumberGenerator : RandomNumberGenerator
        {
            public override String next()
            {
                return "_random";
            }
        }

        private MockRepository m;
        private LocationInformation location = new LocationInformation(country, "Mumbai");
        private static readonly Country country = new Country("IN", "India", "Indian");
        private UserService userService;
        private DomainNameService domainNameService;
        private SuggestionsUtil utils;
        private RandomNumberGenerator randomNumberGenerator;
        private ChildSuggestionFactory childSuggestionFactory;
        private SuggestionStrategyFactory suggestionsFactory;
        private IdentityGenerator identityGenerator;

        [SetUp]
        public void Setup()
        {
            m = new MockRepository();
            userService = m.Stub<UserService>();
            domainNameService = m.Stub<DomainNameService>();
            location = new LocationInformation(country, "Mumbai");
            utils = new SuggestionsUtil(domainNameService);
            randomNumberGenerator = new AnonymousClassRandomNumberGenerator();
            childSuggestionFactory = new ChildSuggestionFactory(userService, utils, randomNumberGenerator);
            suggestionsFactory = new SuggestionStrategyFactory(childSuggestionFactory);
            identityGenerator = new IdentityGenerator(suggestionsFactory);
        }

        [Test]
        public virtual void generateIdsUsingNameLocationAndNationality()
        {
            Expect.Call(domainNameService.isCelebrityName("Naresh", "Jain")).Return(false);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Naresh")).Return(null);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("Jain")).Return(null);
            Expect.Call(userService.isIdentityAvailable("naresh@jain.com")).Return(true);

            Expect.Call(domainNameService.isCelebrityName("Naresh", "India")).Return(false);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Naresh")).Return(null);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("India")).Return(null);
            Expect.Call(userService.isIdentityAvailable("naresh@india.com")).Return(true);

            Expect.Call(domainNameService.isCelebrityName("Naresh", "Indian")).Return(false);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Naresh")).Return(null);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("Indian")).Return(null);
            Expect.Call(userService.isIdentityAvailable("naresh@indian.com")).Return(true);

            Expect.Call(domainNameService.isCelebrityName("Naresh", "Mumbai")).Return(false);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Naresh")).Return(null);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("Mumbai")).Return(null);
            Expect.Call(userService.isIdentityAvailable("naresh@mumbai.com")).Return(true);

            m.ReplayAll();

            SuggestionParam suggestionParam = new SuggestionParam(location, "Naresh", "Jain");

            List<string> generatedIDs = identityGenerator.getGeneratedIDs(suggestionParam);
            List<string> expectedIds =
                ids("naresh@jain.com", "naresh@india.com", "naresh@indian.com", "naresh@mumbai.com");
            Assert.AreEqual(expectedIds, generatedIDs);

            m.VerifyAll();
        }

        [Test]
        public virtual void avoidRestrictedWordsInIds()
        {
            Expect.Call(domainNameService.isCelebrityName("Naresh", "Jain")).Return(false);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Naresh")).Return("Naresh").
                Repeat.Any();

            Expect.Call(domainNameService.isCelebrityName("Nares", "Jain")).Return(false);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Nares")).Return(null);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("Jain")).Return(null);
            Expect.Call(userService.isIdentityAvailable("nares@jain.com")).Return(true);

            Expect.Call(domainNameService.isCelebrityName("Nares", "India")).Return(false);
            Expect.Call(domainNameService.isCelebrityName("Naresh", "India")).Return(false);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Nares")).Return(null);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("India")).Return(null);
            Expect.Call(userService.isIdentityAvailable("nares@india.com")).Return(true);

            Expect.Call(domainNameService.isCelebrityName("Nares", "Indian")).Return(false);
            Expect.Call(domainNameService.isCelebrityName("Naresh", "Indian")).Return(false);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Nares")).Return(null);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("Indian")).Return(null);
            Expect.Call(userService.isIdentityAvailable("nares@indian.com")).Return(true);

            Expect.Call(domainNameService.isCelebrityName("Nares", "Mumbai")).Return(false);
            Expect.Call(domainNameService.isCelebrityName("Naresh", "Mumbai")).Return(false);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Nares")).Return(null);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("Mumbai")).Return(null);
            Expect.Call(userService.isIdentityAvailable("nares@mumbai.com")).Return(true);

            m.ReplayAll();

            SuggestionParam suggestionParam = new SuggestionParam(location, "Naresh", "Jain");
            List<string> generatedIDs = identityGenerator.getGeneratedIDs(suggestionParam);
            List<string> expectedIds = ids("nares@jain.com", "nares@india.com", "nares@indian.com", "nares@mumbai.com");
            Assert.AreEqual(expectedIds, generatedIDs);

            m.VerifyAll();
        }

        [Test]
        public virtual void avoidCelebrityNamesInGeneratedIds()
        {
            Expect.Call(domainNameService.isCelebrityName("Naresh", "Jain")).Return(true);

            Expect.Call(domainNameService.isCelebrityName("Nares", "Jai")).Return(false);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Nares")).Return(null);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("Jai")).Return(null);
            Expect.Call(userService.isIdentityAvailable("nares@jai.com")).Return(true);

            Expect.Call(domainNameService.isCelebrityName("Naresh", "India")).Return(false);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Naresh")).Return(null);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("India")).Return(null);
            Expect.Call(userService.isIdentityAvailable("naresh@india.com")).Return(true);


            Expect.Call(domainNameService.isCelebrityName("Naresh", "Indian")).Return(false);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Naresh")).Return(null);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("Indian")).Return(null);
            Expect.Call(userService.isIdentityAvailable("naresh@indian.com")).Return(true);

            Expect.Call(domainNameService.isCelebrityName("Naresh", "Mumbai")).Return(false);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Naresh")).Return(null);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("Mumbai")).Return(null);
            Expect.Call(userService.isIdentityAvailable("naresh@mumbai.com")).Return(true);

            m.ReplayAll();

            SuggestionParam suggestionParam = new SuggestionParam(location, "Naresh", "Jain");


            List<string> generatedIDs = identityGenerator.getGeneratedIDs(suggestionParam);
            List<string> expectedIds =
                ids("nares@jai.com", "naresh@india.com", "naresh@indian.com", "naresh@mumbai.com");
            Assert.AreEqual(expectedIds, generatedIDs);

            m.VerifyAll();
        }

        [Test]
        public virtual void appendCurrentYearWithFirstNameIfIdIsNotAvailable()
        {
            Expect.Call(domainNameService.isCelebrityName("Naresh", "Jain")).Return(false);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Naresh")).Return(null);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("Jain")).Return(null);
            Expect.Call(userService.isIdentityAvailable("naresh@jain.com")).Return(false);

            Expect.Call(domainNameService.isCelebrityName("Naresh2009", "Jain")).Return(false);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Naresh2009")).Return(null);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("Jain")).Return(null);
            Expect.Call(userService.isIdentityAvailable("naresh2009@jain.com")).Return(true);

            Expect.Call(domainNameService.isCelebrityName("Naresh", "India")).Return(false);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Naresh")).Return(null);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("India")).Return(null);
            Expect.Call(userService.isIdentityAvailable("naresh@india.com")).Return(true);

            Expect.Call(domainNameService.isCelebrityName("Naresh", "Indian")).Return(false);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Naresh")).Return(null);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("Indian")).Return(null);
            Expect.Call(userService.isIdentityAvailable("naresh@indian.com")).Return(true);

            Expect.Call(domainNameService.isCelebrityName("Naresh", "Mumbai")).Return(false);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Naresh")).Return(null);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("Mumbai")).Return(null);
            Expect.Call(userService.isIdentityAvailable("naresh@mumbai.com")).Return(true);

            m.ReplayAll();

            SuggestionParam suggestionParam = new SuggestionParam(location, "Naresh", "Jain");


            List<string> generatedIDs = identityGenerator.getGeneratedIDs(suggestionParam);
            List<string> expectedIds =
                ids("naresh2009@jain.com", "naresh@india.com", "naresh@indian.com", "naresh@mumbai.com");
            Assert.AreEqual(expectedIds, generatedIDs);

            m.VerifyAll();
        }

        [Test]
        public virtual void appendRandomNumberWithFirstNameIfIdIsNotAvailable()
        {
            Expect.Call(domainNameService.isCelebrityName("Naresh", "Jain")).Return(false);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Naresh")).Return(null);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("Jain")).Return(null);
            Expect.Call(userService.isIdentityAvailable("naresh@jain.com")).Return(false);

            Expect.Call(domainNameService.isCelebrityName("Naresh2009", "Jain")).Return(false);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Naresh2009")).Return(null);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("Jain")).Return(null);
            Expect.Call(userService.isIdentityAvailable("naresh2009@jain.com")).Return(false);

            Expect.Call(domainNameService.isCelebrityName("Naresh_random", "Jain")).Return(false);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Naresh_random")).Return(null);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("Jain")).Return(null);
            Expect.Call(userService.isIdentityAvailable("naresh_random@jain.com")).Return(true);

            Expect.Call(domainNameService.isCelebrityName("Naresh", "India")).Return(false);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Naresh")).Return(null);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("India")).Return(null);
            Expect.Call(userService.isIdentityAvailable("naresh@india.com")).Return(true);

            Expect.Call(domainNameService.isCelebrityName("Naresh", "Indian")).Return(false);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Naresh")).Return(null);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("Indian")).Return(null);
            Expect.Call(userService.isIdentityAvailable("naresh@indian.com")).Return(true);

            Expect.Call(domainNameService.isCelebrityName("Naresh", "Mumbai")).Return(false);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny("Naresh")).Return(null);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny("Mumbai")).Return(null);
            Expect.Call(userService.isIdentityAvailable("naresh@mumbai.com")).Return(true);

            m.ReplayAll();

            SuggestionParam suggestionParam = new SuggestionParam(location, "Naresh", "Jain");


            List<string> generatedIDs = identityGenerator.getGeneratedIDs(suggestionParam);
            List<string> expectedIds =
                ids("naresh_random@jain.com", "naresh@india.com", "naresh@indian.com", "naresh@mumbai.com");
            Assert.AreEqual(expectedIds, generatedIDs);

            m.VerifyAll();
        }

        private static List<String> ids(params String[] ids)
        {
            List<string> list = new List<string>();
            list.AddRange(ids);
            return list;
        }
    }
}