using System;
using com.agilefaqs.rft.domain;
using NUnit.Framework;
using Rhino.Mocks;

namespace com.agilefaqs.rft.suggestions
{
    [TestFixture]
    public class SuggestionsUtilTest
    {
        private readonly String token1 = "first_name";
        private readonly String token2 = "last_name";
        private MockRepository m;
        private DomainNameService domainNameService;
        private SuggestionsUtil utils;

        [SetUp]
        public void setUp()
        {
            m = new MockRepository();
            domainNameService = m.StrictMock<DomainNameService>();
            utils = new SuggestionsUtil(domainNameService);
        }

        public void stopRecordingAndStartReplay()
        {
            m.ReplayAll();
        }

        [TearDown]
        public void verifyMocks()
        {
            m.VerifyAll();
        }

        [Test]
        public virtual void shouldGenerateValidIdentityTokensBasedOnPassedTokensIfBothAreNotRestricted()
        {
            Expect.Call(domainNameService.isCelebrityName(token1, token2)).Return(false);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(token1)).Return(null);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(token2)).Return(null);
            Expect.Call(domainNameService.isCelebrityName(token1, token2)).Return(false);
            stopRecordingAndStartReplay();
            IDTokens IdTokens = utils.getIdentityTokens(token1, token2);
            String generatedToken1 = IdTokens.Token1;
            String generatedToken2 = IdTokens.Token2;
            Assert.AreEqual(generatedToken1, token1);
            Assert.AreEqual(generatedToken2, token2);
        }

        [Test]
        public virtual void shouldGenerateValidIdentityTokensWhenTheFirstTokenAreRestricted()
        {
            String expectedToken1 = "first_nam";
            Expect.Call(domainNameService.isCelebrityName(token1, token2)).Return(false);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(token1)).Return(token1);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(expectedToken1)).Return(null);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(token2)).Return(null);
            Expect.Call(domainNameService.isCelebrityName(expectedToken1, token2)).Return(false);
            stopRecordingAndStartReplay();
            IDTokens IdTokens = utils.getIdentityTokens(token1, token2);

            String generatedToken1 = IdTokens.Token1;
            String generatedToken2 = IdTokens.Token2;
            Assert.AreEqual(generatedToken1,
                            expectedToken1);
            Assert.AreEqual(generatedToken2, token2);
        }

        [Test]
        public virtual void shouldGenerateValidIdentityTokensWhenTheFirstTokenIsRestrictedAndLoopThrice()
        {
            String attempt1Token = "first_nam";

            String attempt2Token = "first_na";

            String attempt3Token = "first_n";
            Expect.Call(domainNameService.isCelebrityName(token1, token2)).Return(false);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(token1)).Return(token1);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(attempt1Token)).Return(
                attempt1Token);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(attempt2Token)).Return(
                attempt2Token);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(attempt3Token)).Return(null);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(token2)).Return(null);
            Expect.Call(domainNameService.isCelebrityName(attempt3Token, token2)).Return(false);
            stopRecordingAndStartReplay();
            IDTokens IdTokens = utils.getIdentityTokens(token1, token2);

            String generatedToken1 = IdTokens.Token1;
            String generatedToken2 = IdTokens.Token2;
            Assert.AreEqual(generatedToken1,
                            attempt3Token);
            Assert.AreEqual(generatedToken2, token2);
        }

        [Test]
        public virtual void shouldGenerateValidIdentityTokensWhenTheSecondTokenAreRestricted()
        {
            String expectedToken2 = "last_nam";
            Expect.Call(domainNameService.isCelebrityName(token1, token2)).Return(false);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(token1)).Return(null);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(token2)).Return(token2);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(expectedToken2)).Return(null);
            Expect.Call(domainNameService.isCelebrityName(token1, expectedToken2)).Return(false);
            stopRecordingAndStartReplay();
            IDTokens IdTokens = utils.getIdentityTokens(token1, token2);

            String generatedToken1 = IdTokens.Token1;
            String generatedToken2 = IdTokens.Token2;
            Assert.AreEqual(generatedToken1, token1);
            Assert.AreEqual(generatedToken2,
                            expectedToken2);
        }

        [Test]
        public virtual void shouldGenerateValidIdentityTokensWhenTheSecondTokenIsRestrictedAndLoopThrice()
        {
            String attempt1Token = "last_nam";

            String attempt2Token = "last_na";

            String attempt3Token = "last_n";
            Expect.Call(domainNameService.isCelebrityName(token1, token2)).Return(false);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(token1)).Return(null);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(token2)).Return(token2);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(attempt1Token)).Return(
                attempt1Token);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(attempt2Token)).Return(
                attempt2Token);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(attempt3Token)).Return(null);
            Expect.Call(domainNameService.isCelebrityName(token1, attempt3Token)).Return(false);
            stopRecordingAndStartReplay();
            IDTokens IdTokens = utils.getIdentityTokens(token1, token2);

            String generatedToken1 = IdTokens.Token1;
            String generatedToken2 = IdTokens.Token2;
            Assert.AreEqual(generatedToken1, token1);
            Assert.AreEqual(generatedToken2,
                            attempt3Token);
        }

        [Test]
        public virtual void shouldGenerateValidIdentityTokensBasedOnPassedTokensIfBothAreRestricted()
        {
            String attempt1Token1 = "first_nam";

            String attempt2Token1 = "first_na";

            String attempt3Token1 = "first_n";

            String attempt1Token2 = "last_nam";

            String attempt2Token2 = "last_na";

            String attempt3Token2 = "last_n";
            Expect.Call(domainNameService.isCelebrityName(token1, token2)).Return(false);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(token1)).Return(token1);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(attempt1Token1)).Return(
                attempt1Token1);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(attempt2Token1)).Return(
                attempt2Token1);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(attempt3Token1)).Return(null);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(token2)).Return(token2);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(attempt1Token2)).Return(
                attempt1Token2);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(attempt2Token2)).Return(
                attempt2Token2);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(attempt3Token2)).Return(null);
            Expect.Call(domainNameService.isCelebrityName(attempt3Token1, attempt3Token2)).Return(false);
            stopRecordingAndStartReplay();
            IDTokens IdTokens = utils.getIdentityTokens(token1, token2);

            String generatedToken1 = IdTokens.Token1;
            String generatedToken2 = IdTokens.Token2;
            Assert.AreEqual(generatedToken1,
                            attempt3Token1);
            Assert.AreEqual(generatedToken2,
                            attempt3Token2);
        }

        [Test]
        public virtual void generateIdTokensShouldReturnNullIfMaxNumberOfAttemptsAreMadeForFirstToken()
        {
            String attempt1Token = "first_nam";

            String attempt2Token = "first_na";

            String attempt3Token = "first_n";

            String attempt4Token = "first_";
            Expect.Call(domainNameService.isCelebrityName(token1, token2)).Return(false);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(token1)).Return(token1);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(attempt1Token)).Return(
                attempt1Token);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(attempt2Token)).Return(
                attempt2Token);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(attempt3Token)).Return(
                attempt3Token);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(attempt4Token)).Return(
                attempt4Token);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(token2)).Return(null);
            stopRecordingAndStartReplay();
            IDTokens IdTokens = utils.getIdentityTokens(token1, token2);
            Assert.IsNull(IdTokens);
        }

        [Test]
        public virtual void generateIdTokensShouldReturnNullIfMaxNumberOfAttemptsAreMadeForSecondToken()
        {
            String attempt1Token = "last_nam";

            String attempt2Token = "last_na";

            String attempt3Token = "last_n";

            String attempt4Token = "last_";
            Expect.Call(domainNameService.isCelebrityName(token1, token2)).Return(false);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(token1)).Return(null);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(token2)).Return(token2);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(attempt1Token)).Return(
                attempt1Token);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(attempt2Token)).Return(
                attempt2Token);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(attempt3Token)).Return(
                attempt3Token);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(attempt4Token)).Return(
                attempt4Token);
            stopRecordingAndStartReplay();
            IDTokens IdTokens = utils.getIdentityTokens(token1, token2);
            Assert.IsNull(IdTokens);
        }

        [Test]
        public virtual void generateIdTokensShouldReturnASuggestionIfCombinationOFTokensIsACeleBrity()
        {
            String attempt1Token1 = "first_nam";

            String attempt1Token2 = "last_nam";
            Expect.Call(domainNameService.isCelebrityName(token1, token2)).Return(true);
            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(attempt1Token1)).Return(null);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(attempt1Token2)).Return(null);
            Expect.Call(domainNameService.isCelebrityName(attempt1Token1, attempt1Token2)).Return(false);
            stopRecordingAndStartReplay();

            IDTokens IdTokens = utils.getIdentityTokens(token1, token2);

            String generatedToken1 = IdTokens.Token1;
            String generatedToken2 = IdTokens.Token2;
            Assert.AreEqual(generatedToken1,
                            attempt1Token1);
            Assert.AreEqual(generatedToken2,
                            attempt1Token2);
        }

        [Test]
        public virtual void generateIdTokensShouldReturnNullIfCombinationOFTokensIsACeleBrityAndFiveAttempts()
        {
            String attempt1Token1 = "first_nam";

            String attempt2Token1 = "first_na";

            String attempt3Token1 = "first_n";

            String attempt4Token1 = "first_";

            String attempt1Token2 = "last_nam";

            String attempt2Token2 = "last_na";

            String attempt3Token2 = "last_n";

            String attempt4Token2 = "last_";
            Expect.Call(domainNameService.isCelebrityName(token1, token2)).Return(true);

            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(attempt1Token1)).Return(null);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(attempt1Token2)).Return(null);
            Expect.Call(domainNameService.isCelebrityName(attempt1Token1, attempt1Token2)).Return(true);

            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(attempt2Token1)).Return(null);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(attempt2Token2)).Return(null);
            Expect.Call(domainNameService.isCelebrityName(attempt2Token1, attempt2Token2)).Return(true);

            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(attempt3Token1)).Return(null);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(attempt3Token2)).Return(null);
            Expect.Call(domainNameService.isCelebrityName(attempt3Token1, attempt3Token2)).Return(true);

            Expect.Call(domainNameService.validateFirstPartAndReturnRestrictedWordIfAny(attempt4Token1)).Return(null);
            Expect.Call(domainNameService.validateSecondPartAndReturnRestrictedWordIfAny(attempt4Token2)).Return(null);
            Expect.Call(domainNameService.isCelebrityName(attempt4Token1, attempt4Token2)).Return(true);
            stopRecordingAndStartReplay();

            IDTokens IdTokens = utils.getIdentityTokens(token1, token2);
            Assert.IsNull(IdTokens);
        }
    }
}